package main

import (
	"encoding/json"
	redisSentinel "github.com/go-redis/redis/v7"
	"github.com/gomodule/redigo/redis"
	"log"
	"strings"
)

func getRedisMasterAddr(sentinelAddr, masterName string) string {
	sentinel := redisSentinel.NewSentinelClient(&redisSentinel.Options{
		Addr: sentinelAddr,
	})
	defer sentinel.Close()

	redisMasterAddr, err := sentinel.GetMasterAddrByName(masterName).Result()
	if err != nil {
		log.Fatal(err)
	}

	if len(redisMasterAddr) < 2 {
		log.Fatal("Error obtaining redis master")
	}

	return redisMasterAddr[0] + ":" + redisMasterAddr[1]
}

func initialPicture(platformName string, redisMasterAddr string, c chan TrunksEvent) {
	conn, err := redis.Dial("tcp", redisMasterAddr, redis.DialDatabase(1))
	if err != nil {
		log.Fatal(err)
	}
	defer conn.Close()

	myKeys, err := redis.Strings(conn.Do("KEYS", "trunks:*"))
	if err != nil {
		log.Fatal(err)
	}

	for _, myKey := range myKeys {
		value, err := redis.Bytes(conn.Do("GET", myKey))
		if err != nil || !strings.HasPrefix(myKey, "trunks:") {
			continue
		}

		var trunksDialog TrunksEvent
		err = json.Unmarshal(value, &trunksDialog)
		if err != nil {
			log.Fatal(err)
		}
		trunksDialog.Platform = platformName
		c <- trunksDialog
	}

	return
}

func subscribeForUpdates(platformName string, redisMasterAddr string, c chan TrunksEvent) {
	conn, err := redis.Dial("tcp", redisMasterAddr, redis.DialDatabase(1))
	if err != nil {
		log.Fatal(err)
	}
	defer conn.Close()

	psc := redis.PubSubConn{Conn: conn}
	err = psc.PSubscribe("trunks:*")
	if err != nil {
		log.Fatal(err)
	}
	for {
		switch v := psc.Receive().(type) {
		case redis.Message:
			var trunksDialog TrunksEvent
			err = json.Unmarshal(v.Data, &trunksDialog)
			if err != nil {
				log.Println(err)
			}
			trunksDialog.Platform = platformName
			c <- trunksDialog
		case redis.Subscription:
			continue
		case error:
			log.Println(v)
		}
	}
}
